@@ -23,6 +23,8 @@ class BaseArt(GObject.GObject):
     _CACHE_PATH = GLib.get_user_cache_dir() + "/lollypop"
     # Fallback when album dir is readonly
     _STORE_PATH = GLib.get_user_data_dir() + "/lollypop/store"
+    # Store for Web
+    _WEB_PATH = GLib.get_user_data_dir() + "/lollypop/web"
     __gsignals__ = {
         "album-artwork-changed": (GObject.SignalFlags.RUN_FIRST, None, (int,)),
         "artist-artwork-changed": (GObject.SignalFlags.RUN_FIRST,
@@ -83,52 +85,55 @@ class BaseArt(GObject.GObject):
 #######################
 # PROTECTED           #
 #######################
-    def _preserve_ratio(self, pixbuf, w, h):
+    def _crop_pixbuf(self, pixbuf):
         """
-            Return scaled pixbuf if needed
-            @param uri as str
-            @param w as int
-            @param h as int
+            Return croped pixbuf if needed
+            @param pixbuf as GdkPixbuf.Pixbuf
             @return GdkPixbuf.Pixbuf
         """
-        if App().settings.get_value("preserve-aspect-ratio"):
-            return pixbuf
         width = pixbuf.get_width()
         height = pixbuf.get_height()
         if width == height:
-            preserve = True
-        elif width < height:
-            cut = height / 5
-            preserve = width < height - cut
-        else:
-            cut = width / 5
-            preserve = height < width - cut
-        if preserve:
             return pixbuf
+        max_value = max(width, height)
+        min_value = min(width, height)
+        diff = max_value - min_value
+        if diff > max_value / 4:
+            return pixbuf
+        pixbuf = pixbuf.scale_simple(width + diff,
+                                     height + diff,
+                                     GdkPixbuf.InterpType.BILINEAR)
+        new_pixbuf = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB,
+                                          True, 8, max_value, max_value)
+        if width > height:
+            new_x = diff // 2
+            new_y = 0
+            new_width = max_value
+            new_height = max_value - new_y
         else:
-            return pixbuf.scale_simple(w, h, GdkPixbuf.InterpType.BILINEAR)
+            new_x = 0
+            new_y = diff // 2
+            new_height = max_value
+            new_width = max_value - new_x
+        pixbuf.copy_area(new_x,
+                         new_y,
+                         new_width,
+                         new_height,
+                         new_pixbuf,
+                         0, 0)
+        return new_pixbuf
 
-    def _create_store(self):
+    def _create_dir(self, path):
         """
             Create store dir
+            @param path as str
         """
-        d = Gio.File.new_for_path(self._STORE_PATH)
+        d = Gio.File.new_for_path(path)
         if not d.query_exists():
             try:
                 d.make_directory_with_parents()
             except:
-                Logger.info("Can't create %s" % self._STORE_PATH)
-
-    def _create_cache(self):
-        """
-            Create cache dir
-        """
-        d = Gio.File.new_for_path(self._CACHE_PATH)
-        if not d.query_exists():
-            try:
-                d.make_directory_with_parents()
-            except:
-                Logger.info("Can't create %s" % self._CACHE_PATH)
+                Logger.info("Can't create %s" % path)
 
 #######################
 # PRIVATE             #
