@@ -16,7 +16,6 @@ from gi.repository import GLib, GObject, Gio
 from gi.repository.Gio import FILE_ATTRIBUTE_STANDARD_NAME, \
                               FILE_ATTRIBUTE_STANDARD_TYPE, \
                               FILE_ATTRIBUTE_STANDARD_IS_HIDDEN,\
-                              FILE_ATTRIBUTE_TIME_CHANGED,\
                               FILE_ATTRIBUTE_TIME_MODIFIED
 
 from gettext import gettext as _
@@ -24,8 +23,7 @@ from threading import Thread
 from time import time
 
 from lollypop.inotify import Inotify
-from lollypop.define import App, Type, ScanType
-from lollypop.objects import Track, Album
+from lollypop.define import App, ScanType
 from lollypop.sqlcursor import SqlCursor
 from lollypop.tagreader import TagReader
 from lollypop.logger import Logger
@@ -33,11 +31,10 @@ from lollypop.database_history import History
 from lollypop.utils import is_audio, is_pls, get_mtime, profile
 
 
-SCAN_QUERY_INFO = "{},{},{},{},{}".format(FILE_ATTRIBUTE_STANDARD_NAME,
-                                          FILE_ATTRIBUTE_STANDARD_TYPE,
-                                          FILE_ATTRIBUTE_STANDARD_IS_HIDDEN,
-                                          FILE_ATTRIBUTE_TIME_CHANGED,
-                                          FILE_ATTRIBUTE_TIME_MODIFIED)
+SCAN_QUERY_INFO = "{},{},{},{}".format(FILE_ATTRIBUTE_STANDARD_NAME,
+                                       FILE_ATTRIBUTE_STANDARD_TYPE,
+                                       FILE_ATTRIBUTE_STANDARD_IS_HIDDEN,
+                                       FILE_ATTRIBUTE_TIME_MODIFIED)
 
 
 class CollectionScanner(GObject.GObject, TagReader):
@@ -59,7 +56,7 @@ class CollectionScanner(GObject.GObject, TagReader):
         TagReader.__init__(self)
 
         self.__thread = None
-        self.__history = None
+        self.__history = History()
         self.__disable_compilations = True
         if App().settings.get_value("auto-update"):
             self.__inotify = Inotify()
@@ -95,6 +92,104 @@ class CollectionScanner(GObject.GObject, TagReader):
             self.__thread.daemon = True
             self.__thread.start()
 
+    def update_album(self, album_id, album_artist_ids,
+                     genre_ids, year, timestamp):
+        """
+            Update album artists based on album-artist and artist tags
+            This code auto handle compilations: empty "album artist" with
+            different artists
+            @param album_id as int
+            @param album_artist_ids as [int]
+            @param genre_ids as [int]
+            @param year as int
+            @param timestmap as int
+            @commit needed
+        """
+        if album_artist_ids:
+            # Update UI based on previous artist calculation
+            mtime = App().albums.get_mtime(album_id)
+            if mtime != 0:
+                for artist_id in album_artist_ids:
+                    GLib.idle_add(self.emit, "artist-updated", artist_id, True)
+            App().albums.set_artist_ids(album_id, album_artist_ids)
+        # Set artist ids based on content
+        else:
+            new_album_artist_ids = App().albums.calculate_artist_ids(album_id)
+            App().albums.set_artist_ids(album_id, new_album_artist_ids)
+        # Update album genres
+        for genre_id in genre_ids:
+            App().albums.add_genre(album_id, genre_id)
+        # Update year based on tracks
+        year = App().tracks.get_year_for_album(album_id)
+        App().albums.set_year(album_id, year)
+        timestamp = App().tracks.get_timestamp_for_album(album_id)
+        App().albums.set_timestamp(album_id, timestamp)
+
+    def update_track(self, track_id, artist_ids, genre_ids):
+        """
+            Set track artists/genres
+            @param track id as int
+            @param artist ids as [int]
+            @param genre ids as [int]
+            @param mtime as int
+            @param popularity as int
+            @commit needed
+        """
+        # Set artists/genres for track
+        for artist_id in artist_ids:
+            App().tracks.add_artist(track_id, artist_id)
+        for genre_id in genre_ids:
+            App().tracks.add_genre(track_id, genre_id)
+
+    def del_from_db(self, uri, backup, notify=True):
+        """
+            Delete track from db
+            @param uri as str
+            @param backup as bool
+            @param notify as bool => send signal about cleanup
+            @return (popularity, ltime, mtime,
+                     loved album, album_popularity)
+        """
+        try:
+            track_id = App().tracks.get_id_by_uri(uri)
+            duration = App().tracks.get_duration(track_id)
+            album_id = App().tracks.get_album_id(track_id)
+            genre_ids = App().tracks.get_genre_ids(track_id)
+            album_artist_ids = App().albums.get_artist_ids(album_id)
+            artist_ids = App().tracks.get_artist_ids(track_id)
+            track_pop = App().tracks.get_popularity(track_id)
+            track_rate = App().tracks.get_rate(track_id)
+            track_ltime = App().tracks.get_ltime(track_id)
+            album_mtime = App().tracks.get_mtime(track_id)
+            track_loved = App().tracks.get_loved(track_id)
+            album_pop = App().albums.get_popularity(album_id)
+            album_rate = App().albums.get_rate(album_id)
+            album_loved = App().albums.get_loved(album_id)
+            if backup:
+                f = Gio.File.new_for_uri(uri)
+                name = f.get_basename()
+                self.__history.add(name, duration, track_pop, track_rate,
+                                   track_ltime, album_mtime, track_loved,
+                                   album_loved, album_pop, album_rate)
+            App().tracks.remove(track_id)
+            if notify:
+                App().albums.clean()
+                App().genres.clean()
+                App().artists.clean()
+                if App().albums.get_name(album_id) is None:
+                    GLib.idle_add(self.emit, "album-updated",
+                                  album_id, False)
+                for artist_id in album_artist_ids + artist_ids:
+                    GLib.idle_add(self.emit, "artist-updated",
+                                  artist_id, False)
+                for genre_id in genre_ids:
+                    GLib.idle_add(self.emit, "genre-updated",
+                                  genre_id, False)
+            return (track_pop, track_rate, track_ltime, album_mtime,
+                    track_loved, album_loved, album_pop, album_rate)
+        except Exception as e:
+            Logger.error("CollectionScanner::del_from_db: %s" % e)
+
     def is_locked(self):
         """
             Return True if db locked
@@ -201,9 +296,6 @@ class CollectionScanner(GObject.GObject, TagReader):
             @param uris as [str]
             @thread safe
         """
-        if scan_type != ScanType.EPHEMERAL and self.__history is None:
-            self.__history = History()
-
         (files, dirs) = self.__get_objects_for_uris(scan_type, uris)
 
         if scan_type == ScanType.NEW_FILES:
@@ -217,11 +309,7 @@ class CollectionScanner(GObject.GObject, TagReader):
             GLib.idle_add(self.__finish, new_tracks)
 
         if scan_type == ScanType.EPHEMERAL:
-            self.__play_new_tracks(new_tracks)
-
-        if scan_type != ScanType.EPHEMERAL:
-            del self.__history
-            self.__history = None
+            App().player.play_uris(new_tracks)
 
     def __scan_to_handle(self, uri):
         """
@@ -304,7 +392,7 @@ class CollectionScanner(GObject.GObject, TagReader):
                                 break
                     f = Gio.File.new_for_uri(uri)
                     if not in_collection or not f.query_exists():
-                        self.__del_from_db(uri)
+                        self.del_from_db(uri, True)
                         SqlCursor.allow_thread_execution(App().db)
         except Exception as e:
             Logger.warning("CollectionScanner:: __scan_files: % s" % e)
@@ -340,6 +428,7 @@ class CollectionScanner(GObject.GObject, TagReader):
         discnumber = self.get_discnumber(tags)
         discname = self.get_discname(tags)
         tracknumber = self.get_tracknumber(tags, name)
+        track_popm = self.get_popm(tags)
         (year, timestamp) = self.get_original_year(tags)
         if year is None:
             (year, timestamp) = self.get_year(tags)
@@ -368,12 +457,7 @@ class CollectionScanner(GObject.GObject, TagReader):
             basename = f.get_basename()
             track_id = App().tracks.get_id_by_basename_duration(basename,
                                                                 duration)
-        # Restore from history
-        if self.__history is None:
-            (track_pop, track_rate, track_ltime,
-             album_mtime, track_loved, album_loved,
-             album_pop, album_rate) = (0, 0, 0, 0, False, False, 0, 0)
-        elif track_id is None:
+        if track_id is None:
             (track_pop, track_rate, track_ltime,
              album_mtime, track_loved, album_loved,
              album_pop, album_rate) = self.__history.get(name, duration)
@@ -381,7 +465,10 @@ class CollectionScanner(GObject.GObject, TagReader):
         else:
             (track_pop, track_rate, track_ltime,
              album_mtime, track_loved, album_loved,
-             album_pop, album_rate) = self.__del_from_db(uri)
+             album_pop, album_rate) = self.del_from_db(uri, False)
+        # Prefer popm to internal rate
+        if track_popm != 0:
+            track_rate = track_popm
         # If nothing in stats, use track mtime
         if album_mtime == 0:
             album_mtime = mtime
@@ -406,11 +493,10 @@ class CollectionScanner(GObject.GObject, TagReader):
 
         Logger.debug("CollectionScanner::add2db(): Add album: "
                      "%s, %s" % (album_name, album_artist_ids))
-        album_id = self.add_album(album_name, mb_album_id,
-                                  album_artist_ids,
-                                  uri, album_loved, album_pop,
-                                  album_rate, mtime)
-
+        (album_added, album_id) = self.add_album(album_name, mb_album_id,
+                                                 album_artist_ids,
+                                                 uri, album_loved, album_pop,
+                                                 album_rate, mtime)
         genre_ids = self.add_genres(genres)
 
         # Add track to db
@@ -421,140 +507,14 @@ class CollectionScanner(GObject.GObject, TagReader):
                                     track_rate, track_loved, track_ltime,
                                     mtime, mb_track_id)
         Logger.debug("CollectionScanner::add2db(): Update track")
-        self.__update_track(track_id, artist_ids, genre_ids)
+        self.update_track(track_id, artist_ids, genre_ids)
         Logger.debug("CollectionScanner::add2db(): Update album")
         SqlCursor.commit(App().db)
-        self.__update_album(album_id, album_artist_ids,
-                            genre_ids, year, timestamp)
+        self.update_album(album_id, album_artist_ids,
+                          genre_ids, year, timestamp)
         SqlCursor.commit(App().db)
         for genre_id in genre_ids:
             GLib.idle_add(self.emit, "genre-updated", genre_id, True)
+        if album_added:
+            GLib.idle_add(self.emit, "album-updated", album_id, True)
         return track_id
-
-    def __del_from_db(self, uri):
-        """
-            Delete track from db
-            @param uri as str
-            @return (popularity, ltime, mtime,
-                     loved album, album_popularity)
-        """
-        try:
-            track_id = App().tracks.get_id_by_uri(uri)
-            album_id = App().tracks.get_album_id(track_id)
-            genre_ids = App().tracks.get_genre_ids(track_id)
-            album_artist_ids = App().albums.get_artist_ids(album_id)
-            artist_ids = App().tracks.get_artist_ids(track_id)
-            track_pop = App().tracks.get_popularity(track_id)
-            track_rate = App().tracks.get_rate(track_id)
-            track_ltime = App().tracks.get_ltime(track_id)
-            album_mtime = App().tracks.get_mtime(track_id)
-            track_loved = App().tracks.get_loved(track_id)
-            album_pop = App().albums.get_popularity(album_id)
-            album_rate = App().albums.get_rate(album_id)
-            album_loved = App().albums.get_loved(album_id)
-            uri = App().tracks.get_uri(track_id)
-            App().tracks.remove(track_id)
-            App().tracks.clean(track_id)
-            cleaned = App().albums.clean(album_id)
-            if cleaned:
-                SqlCursor.commit(App().db)
-                GLib.idle_add(self.emit, "album-updated",
-                              album_id, True)
-            for artist_id in album_artist_ids + artist_ids:
-                cleaned = App().artists.clean(artist_id)
-                if cleaned:
-                    GLib.idle_add(self.emit, "artist-updated",
-                                  artist_id, False)
-            for genre_id in genre_ids:
-                cleaned = App().genres.clean(genre_id)
-                if cleaned:
-                    SqlCursor.commit(App().db)
-                    GLib.idle_add(self.emit, "genre-updated",
-                                  genre_id, False)
-            return (track_pop, track_rate, track_ltime, album_mtime,
-                    track_loved, album_loved, album_pop, album_rate)
-        except Exception as e:
-            Logger.error("CollectionScanner::__del_from_db: %s" % e)
-
-    def __update_album(self, album_id, artist_ids, genre_ids, year, timestamp):
-        """
-            Update album artists based on album-artist and artist tags
-            This code auto handle compilations: empty "album artist" with
-            different artists
-            @param album id as int
-            @param artist ids as [int]
-            @param genre ids as [int]
-            @param year as int
-            @param timestmap as int
-            @commit needed
-        """
-        album_artist_ids = []
-        add = True
-        # Set artist ids based on content
-        if not artist_ids:
-            new_artist_ids = App().albums.calculate_artist_ids(album_id)
-            current_artist_ids = App().albums.get_artist_ids(album_id)
-            if new_artist_ids != current_artist_ids:
-                album_artist_ids = new_artist_ids
-                if Type.COMPILATIONS in new_artist_ids:
-                    add = False
-                    album_artist_ids = current_artist_ids
-                else:
-                    album_artist_ids = new_artist_ids
-                App().albums.set_artist_ids(album_id, new_artist_ids)
-        # Update UI based on previous artist calculation
-        if App().albums.get_tracks_count(album_id) > 1:
-            for artist_id in album_artist_ids:
-                GLib.idle_add(self.emit, "artist-updated", artist_id, add)
-        # Update album genres
-        for genre_id in genre_ids:
-            App().albums.add_genre(album_id, genre_id)
-
-        # Update year based on tracks
-        year = App().tracks.get_year_for_album(album_id)
-        App().albums.set_year(album_id, year)
-        timestamp = App().tracks.get_timestamp_for_album(album_id)
-        App().albums.set_timestamp(album_id, timestamp)
-
-    def __update_track(self, track_id, artist_ids, genre_ids):
-        """
-            Set track artists/genres
-            @param track id as int
-            @param artist ids as [int]
-            @param genre ids as [int]
-            @param mtime as int
-            @param popularity as int
-            @commit needed
-        """
-        # Set artists/genres for track
-        for artist_id in artist_ids:
-            App().tracks.add_artist(track_id, artist_id)
-        for genre_id in genre_ids:
-            App().tracks.add_genre(track_id, genre_id)
-
-    def __play_new_tracks(self, uris):
-        """
-            Play new tracks
-            @param uri as [str]
-        """
-        # First get tracks
-        tracks = []
-        for uri in uris:
-            track_id = App().tracks.get_id_by_uri(uri)
-            tracks.append(Track(track_id))
-        # Then get album ids
-        album_ids = {}
-        for track in tracks:
-            if track.album.id in album_ids.keys():
-                album_ids[track.album.id].append(track)
-            else:
-                album_ids[track.album.id] = [track]
-        # Create albums with tracks
-        play = True
-        for album_id in album_ids.keys():
-            album = Album(album_id)
-            album.set_tracks(album_ids[album_id])
-            if play:
-                App().player.play_album(album)
-            else:
-                App().player.add_album(album)
