@@ -81,29 +81,6 @@ class ListsContainer:
         else:
             self._list_one.select_ids(list_one_ids)
 
-    def show_artists_albums(self, artist_ids):
-        """
-            Show albums from artists
-            @param artist id as int
-        """
-        def select_list_two(selection_list, artist_ids):
-            self._list_two.select_ids(artist_ids)
-            self._list_two.disconnect_by_func(select_list_two)
-        sidebar_content = App().settings.get_enum("sidebar-content")
-        if sidebar_content == SidebarContent.GENRES:
-            # Get artist genres
-            genre_ids = []
-            for artist_id in artist_ids:
-                album_ids = App().artists.get_albums(artist_ids)
-                for album_id in album_ids:
-                    for genre_id in App().albums.get_genre_ids(album_id):
-                        if genre_id not in genre_ids:
-                            genre_ids.append(genre_id)
-            self.show_lists(genre_ids, artist_ids)
-        else:
-            # Select artists on list one
-            self.show_lists(artist_ids, [])
-
     @property
     def list_one(self):
         """
@@ -147,6 +124,14 @@ class ListsContainer:
         if state_two_ids:
             self._list_two.connect("populated", select_list_two, state_two_ids)
         if state_one_ids:
+            # Here we are just handling missing Type.ARTISTS from list
+            if state_one_ids[0] == Type.ARTISTS:
+                from lollypop.shown import ShownLists
+                shown_lists = ShownLists.get(SelectionListMask.LIST_ONE)
+                ids = [l[0] for l in shown_lists]
+                if Type.ARTISTS not in ids:
+                    self._list_one.add_value((Type.ARTISTS,
+                                             _("All artists"), ""))
             self._list_one.select_ids(state_one_ids)
         elif not App().window.is_adaptive:
             self._list_one.select_first()
@@ -265,14 +250,15 @@ class ListsContainer:
         elif selected_ids[0] == Type.CURRENT:
             view = self.get_view_current()
         elif selected_ids[0] == Type.SEARCH:
-            view = self._get_view_search()
+            view = self.get_view_search()
         elif Type.DEVICES - 999 < selected_ids[0] < Type.DEVICES:
             view = self._get_view_device(selected_ids[0])
         elif selected_ids[0] in [Type.POPULARS,
                                  Type.LOVED,
                                  Type.RECENTS,
                                  Type.NEVER,
-                                 Type.RANDOMS]:
+                                 Type.RANDOMS,
+                                 Type.WEB]:
             view = self._get_view_albums(selected_ids, [])
         elif selected_ids[0] == Type.RADIOS:
             view = self._get_view_radios()
