@@ -10,13 +10,12 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gettext import gettext as _
 import itertools
 
 from lollypop.sqlcursor import SqlCursor
 from lollypop.define import App, Type, OrderBy
 from lollypop.logger import Logger
-from lollypop.utils import remove_static_genres, noaccents
+from lollypop.utils import noaccents
 
 
 class AlbumsDatabase:
@@ -63,7 +62,7 @@ class AlbumsDatabase:
     def add_artist(self, album_id, artist_id):
         """
             Add artist to track
-            @param album id as int
+            @param album_id as int
             @param artist id as int
             @warning: commit needed
         """
@@ -77,7 +76,7 @@ class AlbumsDatabase:
     def add_genre(self, album_id, genre_id):
         """
             Add genre to album
-            @param album id as int
+            @param album_id as int
             @param genre id as int
             @warning: commit needed
         """
@@ -92,24 +91,22 @@ class AlbumsDatabase:
     def set_artist_ids(self, album_id, artist_ids):
         """
             Set artist id
-            @param album id as int
+            @param album_id as int
             @param artist_ids as [int]
             @warning: commit needed
         """
         with SqlCursor(App().db, True) as sql:
-            currents = self.get_artist_ids(album_id)
-            if not currents or set(currents) - set(artist_ids):
-                sql.execute("DELETE FROM album_artists\
-                            WHERE album_id=?", (album_id,))
-                for artist_id in artist_ids:
-                    sql.execute("INSERT INTO album_artists\
-                                (album_id, artist_id)\
-                                VALUES (?, ?)", (album_id, artist_id))
+            sql.execute("DELETE FROM album_artists\
+                        WHERE album_id=?", (album_id,))
+            for artist_id in artist_ids:
+                sql.execute("INSERT INTO album_artists\
+                            (album_id, artist_id)\
+                            VALUES (?, ?)", (album_id, artist_id))
 
     def set_synced(self, album_id, synced):
         """
             Set album synced
-            @param album id as int
+            @param album_id as int
             @param synced as int
             @warning: commit needed
         """
@@ -117,10 +114,20 @@ class AlbumsDatabase:
             sql.execute("UPDATE albums SET synced=? WHERE rowid=?",
                         (synced, album_id))
 
+    def set_mtime(self, album_id, mtime):
+        """
+            Set album mtime
+            @param album_id as int
+            @param mtime as int
+        """
+        with SqlCursor(App().db, True) as sql:
+            sql.execute("UPDATE albums SET mtime=? WHERE rowid=?",
+                        (mtime, album_id))
+
     def set_loved(self, album_id, loved):
         """
             Set album loved
-            @param album id as int
+            @param album_id as int
             @param loved as int
             @warning: commit needed
         """
@@ -140,7 +147,7 @@ class AlbumsDatabase:
     def set_year(self, album_id, year):
         """
             Set year
-            @param album id as int
+            @param album_id as int
             @param year as int
             @warning: commit needed
         """
@@ -151,7 +158,7 @@ class AlbumsDatabase:
     def set_timestamp(self, album_id, timestamp):
         """
             Set timestamp
-            @param album id as int
+            @param album_id as int
             @param timestamp as int
             @warning: commit needed
         """
@@ -162,7 +169,7 @@ class AlbumsDatabase:
     def set_uri(self, album_id, uri):
         """
             Set album uri for album id
-            @param Album id as int, uri as string
+            @param album_id as int, uri as string
             @warning: commit needed
         """
         with SqlCursor(App().db, True) as sql:
@@ -192,7 +199,7 @@ class AlbumsDatabase:
                        WHERE album_artists.album_id = albums.rowid\
                        AND (album_artists.artist_id = artists.rowid\
                             OR album_artists.artist_id=?)\
-                       AND synced=1"
+                       AND synced=1 AND albums.mtime != 0"
             order = " ORDER BY artists.sortname\
                      COLLATE NOCASE COLLATE LOCALIZED,\
                      albums.timestamp,\
@@ -220,7 +227,7 @@ class AlbumsDatabase:
     def get_loved(self, album_id):
         """
             Get album loved
-            @param album id as int
+            @param album_id as int
             @return loved as int
         """
         with SqlCursor(App().db) as sql:
@@ -248,7 +255,7 @@ class AlbumsDatabase:
     def get_rate(self, album_id):
         """
             Get album rate
-            @param album id as int
+            @param album_id as int
             @return rate as int
         """
         with SqlCursor(App().db) as sql:
@@ -278,7 +285,7 @@ class AlbumsDatabase:
     def set_more_popular(self, album_id, pop_to_add):
         """
             Increment popularity field for album id
-            @param album id as int
+            @param album_id as int
             @param pop as int
             @raise sqlite3.OperationalError on db update
         """
@@ -357,10 +364,32 @@ class AlbumsDatabase:
                 return v[0]
             return None
 
+    def get_id_by_name_artists(self, album_name, artist_ids):
+        """
+            Get non compilation album id
+            @param album_name as str
+            @param artist_ids as [int]
+            @return int
+        """
+        with SqlCursor(App().db) as sql:
+            filters = (album_name,)
+            request = "SELECT albums.rowid FROM albums, album_artists\
+                       WHERE name=? COLLATE NOCASE AND\
+                       album_artists.album_id=albums.rowid AND (1=0 "
+            filters += tuple(artist_ids)
+            for artist_id in artist_ids:
+                request += "OR artist_id=? "
+            request += ")"
+            result = sql.execute(request, filters)
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return None
+
     def get_genre_ids(self, album_id):
         """
             Get genre ids
-            @param Album id as int
+            @param album_id as int
             @return Genres id as [int]
         """
         with SqlCursor(App().db) as sql:
@@ -371,8 +400,8 @@ class AlbumsDatabase:
     def get_name(self, album_id):
         """
             Get album name for album id
-            @param Album id as int
-            @return Album name as string
+            @param album_id as int
+            @return str
         """
         with SqlCursor(App().db) as sql:
             result = sql.execute("SELECT name FROM albums where rowid=?",
@@ -380,13 +409,12 @@ class AlbumsDatabase:
             v = result.fetchone()
             if v is not None:
                 return v[0]
-
-            return _("Unknown")
+            return None
 
     def get_artists(self, album_id):
         """
             Get artist names
-            @param Album id as int
+            @param album_id as int
             @return artists as [str]
         """
         with SqlCursor(App().db) as sql:
@@ -410,10 +438,24 @@ class AlbumsDatabase:
                                  (album_id,))
             return list(itertools.chain(*result))
 
+    def get_mtime(self, album_id):
+        """
+            Get modification time
+            @param album_id as int
+            @return modification time as int
+        """
+        with SqlCursor(App().db) as sql:
+            request = "SELECT mtime FROM albums WHERE albums.rowid=?"
+            result = sql.execute(request, (album_id,))
+            v = result.fetchone()
+            if v is not None:
+                return v[0]
+            return 0
+
     def get_year(self, album_id):
         """
             Get album year
-            @param album id as int
+            @param album_id as int
             @return album year as int
         """
         with SqlCursor(App().db) as sql:
@@ -427,7 +469,7 @@ class AlbumsDatabase:
     def get_uri(self, album_id):
         """
             Get album uri for album id
-            @param Album id as int
+            @param album_id as int
             @return Album uri as string
         """
         with SqlCursor(App().db) as sql:
@@ -465,7 +507,7 @@ class AlbumsDatabase:
     def get_tracks_count(self, album_id):
         """
             Return tracks count
-            @param album id as int
+            @param album_id as int
             @return count as int
         """
         with SqlCursor(App().db) as sql:
@@ -485,7 +527,8 @@ class AlbumsDatabase:
         """
         with SqlCursor(App().db) as sql:
             request = "SELECT DISTINCT albums.rowid\
-                       FROM albums WHERE rate>=4\
+                       FROM albums\
+                       WHERE rate>=4 AND loved != -1 AND mtime != 0\
                        ORDER BY popularity DESC LIMIT ?"
             result = sql.execute(request, (limit,))
             return list(itertools.chain(*result))
@@ -498,7 +541,7 @@ class AlbumsDatabase:
         """
         with SqlCursor(App().db) as sql:
             request = "SELECT DISTINCT albums.rowid FROM albums\
-                       WHERE popularity!=0 AND albums.loved != -1\
+                       WHERE popularity!=0 AND loved != -1 AND mtime != 0\
                        ORDER BY popularity DESC LIMIT ?"
             result = sql.execute(request, (limit,))
             return list(itertools.chain(*result))
@@ -511,7 +554,8 @@ class AlbumsDatabase:
         with SqlCursor(App().db) as sql:
             request = "SELECT DISTINCT albums.rowid\
                        FROM albums\
-                       WHERE loved=1 ORDER BY popularity DESC"
+                       WHERE loved=1 AND\
+                       mtime != 0 ORDER BY popularity DESC"
             result = sql.execute(request)
             return list(itertools.chain(*result))
 
@@ -522,7 +566,8 @@ class AlbumsDatabase:
         """
         with SqlCursor(App().db) as sql:
             request = "SELECT DISTINCT albums.rowid FROM albums\
-                       WHERE albums.loved != -1\
+                       WHERE albums.loved != -1 AND\
+                       albums.mtime != 0\
                        ORDER BY mtime DESC LIMIT 100"
             result = sql.execute(request)
             return list(itertools.chain(*result))
@@ -535,7 +580,8 @@ class AlbumsDatabase:
         with SqlCursor(App().db) as sql:
             albums = []
             request = "SELECT DISTINCT albums.rowid FROM albums\
-                       WHERE albums.loved != -1 ORDER BY random() LIMIT 100"
+                       WHERE albums.loved != -1 AND\
+                       albums.mtime != 0 ORDER BY random() LIMIT 100"
             result = sql.execute(request)
             albums = list(itertools.chain(*result))
             self._cached_randoms = list(albums)
@@ -550,7 +596,7 @@ class AlbumsDatabase:
     def get_disc_names(self, album_id, disc):
         """
             Get disc names
-            @param album id as int
+            @param album_id as int
             @param disc as int
             @return name as str
         """
@@ -567,11 +613,10 @@ class AlbumsDatabase:
     def get_discs(self, album_id, genre_ids):
         """
             Get disc numbers
-            @param album id as int
+            @param album_id as int
             @param genre id as int
             @return [disc as int]
         """
-        genre_ids = remove_static_genres(genre_ids)
         with SqlCursor(App().db) as sql:
             filters = (album_id,)
             filters += tuple(genre_ids)
@@ -591,7 +636,7 @@ class AlbumsDatabase:
     def get_track_uris(self, album_id):
         """
             Get track uris for album id/disc
-            @param album id as int
+            @param album_id as int
             @return [int]
         """
         with SqlCursor(App().db) as sql:
@@ -605,14 +650,13 @@ class AlbumsDatabase:
         """
             Get tracks ids for album id disc
 
-            @param album id as int
+            @param album_id as int
             @param genre ids as [int]
             @param artist ids as [int]
             @param disc as int
             @param disallow_ignored_tracks as bool
             @return [int]
         """
-        genre_ids = remove_static_genres(genre_ids)
         with SqlCursor(App().db) as sql:
             filters = (album_id, disc)
             request = "SELECT DISTINCT tracks.rowid\
@@ -665,7 +709,6 @@ class AlbumsDatabase:
             @param ignore as bool => ignore albums with loved == 1
             @return albums ids as [int]
         """
-        genre_ids = remove_static_genres(genre_ids)
         orderby = App().settings.get_enum("orderby")
         if artist_ids or orderby == OrderBy.ARTIST:
             order = " ORDER BY artists.sortname\
@@ -692,6 +735,7 @@ class AlbumsDatabase:
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_artists, artists\
                            WHERE albums.rowid = album_artists.album_id AND\
+                           albums.mtime!=0 AND\
                            artists.rowid = album_artists.artist_id"
                 if ignore:
                     request += " AND albums.loved != -1"
@@ -704,6 +748,7 @@ class AlbumsDatabase:
                            album_genres, album_artists, artists\
                            WHERE albums.rowid = album_artists.album_id AND\
                            artists.rowid = album_artists.artist_id AND\
+                           albums.mtime!=0 AND\
                            album_genres.album_id=albums.rowid AND ( "
                 for genre_id in genre_ids:
                     request += "album_genres.genre_id=? OR "
@@ -718,6 +763,7 @@ class AlbumsDatabase:
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_artists, artists\
                            WHERE album_artists.album_id=albums.rowid AND\
+                           albums.mtime!=0 AND\
                            artists.rowid = album_artists.artist_id AND ("
                 for artist_id in artist_ids:
                     request += "artists.rowid=? OR "
@@ -734,6 +780,7 @@ class AlbumsDatabase:
                            FROM albums, album_genres, album_artists, artists\
                            WHERE album_genres.album_id=albums.rowid AND\
                            artists.rowid = album_artists.artist_id AND\
+                           albums.mtime!=0 AND\
                            album_artists.album_id=albums.rowid AND ("
                 for artist_id in artist_ids:
                     request += "artists.rowid=? OR "
@@ -763,6 +810,7 @@ class AlbumsDatabase:
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_artists\
                            WHERE album_artists.artist_id=?\
+                           AND albums.mtime != 0\
                            AND album_artists.album_id=albums.rowid"
                 if ignore:
                     request += " AND albums.loved != -1"
@@ -775,6 +823,7 @@ class AlbumsDatabase:
                 request = "SELECT DISTINCT albums.rowid\
                            FROM albums, album_genres, album_artists\
                            WHERE album_genres.album_id=albums.rowid\
+                           AND albums.mtime != 0\
                            AND albums.loved != -1\
                            AND album_artists.album_id=albums.rowid\
                            AND album_artists.artist_id=? AND ( "
@@ -790,7 +839,7 @@ class AlbumsDatabase:
     def get_duration(self, album_id, genre_ids):
         """
             Album duration in seconds
-            @param album id as int
+            @param album_id as int
             @param genre ids as [int]
             @return album duration as int
         """
@@ -837,6 +886,7 @@ class AlbumsDatabase:
                                   FROM albums, tracks\
                                   WHERE tracks.ltime=0 AND\
                                   albums.loved != -1 AND\
+                                  albums.mtime != 0 AND\
                                   albums.rowid=tracks.album_id\
                                   AND albums.popularity < 10\
                                   ORDER BY random() LIMIT 100")
@@ -868,7 +918,7 @@ class AlbumsDatabase:
             if limit != -1:
                 result = sql.execute("SELECT albums.rowid\
                                       FROM albums\
-                                      WHERE albums.year=?\
+                                      WHERE year=? AND mtime != 0\
                                       ORDER BY random() LIMIT ?",
                                      (year, limit))
             else:
@@ -882,14 +932,14 @@ class AlbumsDatabase:
                                FROM albums, album_artists, artists\
                                WHERE albums.rowid=album_artists.album_id AND\
                                artists.rowid=album_artists.artist_id AND\
-                               albums.year is null"
+                               albums.year is null AND albums.mtime != 0"
                     filter = ()
                 else:
                     request = "SELECT DISTINCT albums.rowid\
                                FROM albums, album_artists, artists\
                                WHERE albums.rowid=album_artists.album_id AND\
                                artists.rowid=album_artists.artist_id AND\
-                               albums.year=?"
+                               albums.year=? AND albums.mtime != 0"
                     filter = (year,)
                 request += order
                 result = sql.execute(request, filter)
@@ -907,6 +957,7 @@ class AlbumsDatabase:
                                       FROM albums, album_artists\
                                       WHERE album_artists.artist_id=?\
                                       AND album_artists.album_id=albums.rowid\
+                                      AND albums.mtime != 0\
                                       AND albums.year=? LIMIT ?",
                                      (Type.COMPILATIONS, year, limit))
             else:
@@ -917,6 +968,7 @@ class AlbumsDatabase:
                                FROM albums, album_artists\
                                WHERE album_artists.artist_id=?\
                                AND album_artists.album_id=albums.rowid\
+                               AND albums.mtime != 0\
                                AND albums.year is null"
                     filter = (Type.COMPILATIONS,)
                 else:
@@ -924,6 +976,7 @@ class AlbumsDatabase:
                                FROM albums, album_artists\
                                WHERE album_artists.artist_id=?\
                                AND album_artists.album_id=albums.rowid\
+                               AND albums.mtime != 0\
                                AND albums.year=?"
                     filter = (Type.COMPILATIONS, year)
                 request += order
@@ -942,10 +995,10 @@ class AlbumsDatabase:
             for filter in [(no_accents + "%",),
                            ("%" + no_accents,),
                            ("%" + no_accents + "%",)]:
-                result = sql.execute("SELECT albums.rowid\
-                                      FROM albums\
-                                      WHERE noaccents(name) LIKE ? LIMIT 25",
-                                     filter)
+                request = "SELECT albums.rowid FROM albums\
+                           WHERE noaccents(name) LIKE ?\
+                           AND albums.mtime!=0 LIMIT 25"
+                result = sql.execute(request, filter)
                 items += list(itertools.chain(*result))
             return items
 
@@ -953,7 +1006,7 @@ class AlbumsDatabase:
         """
             Calculate artist ids based on tracks
             @WARNING Be sure album already have a track
-            @param album id as int
+            @param album_id as int
             @return artist_ids as [int]
         """
         ret = []
@@ -986,44 +1039,19 @@ class AlbumsDatabase:
                 return v[0]
             return 0
 
-    def clean(self, album_id):
+    def clean(self):
         """
-            Clean database for album id
-            @param album id as int
-            @param return True if album deleted or genre modified
-            @warning commit needed
+            Clean albums
         """
         with SqlCursor(App().db, True) as sql:
-            ret = False
-            # Check album really have tracks from its genres
-            for genre_id in self.get_genre_ids(album_id):
-                result = sql.execute("SELECT track_id\
-                                      FROM tracks, track_genres\
-                                      WHERE track_genres.track_id ==\
-                                      tracks.rowid\
-                                      AND tracks.album_id=?\
-                                      AND track_genres.genre_id=?",
-                                     (album_id, genre_id))
-                v = result.fetchone()
-                if not v:
-                    ret = True
-                    sql.execute("DELETE from album_genres\
-                                 WHERE album_id=?\
-                                 AND genre_id=?", (album_id, genre_id))
-
-            # Remove album if orphaned
-            result = sql.execute("SELECT rowid from tracks\
-                                  WHERE album_id=?\
-                                  LIMIT 1", (album_id,))
-            v = result.fetchone()
-            # Album empty, remove it
-            if not v:
-                ret = True
-                sql.execute("DELETE FROM album_artists\
-                            WHERE album_id=?",
-                            (album_id,))
-                sql.execute("DELETE FROM albums WHERE rowid=?", (album_id,))
-            return ret
+            sql.execute("DELETE FROM albums WHERE albums.rowid NOT IN (\
+                            SELECT tracks.album_id FROM tracks)")
+            sql.execute("DELETE FROM album_genres\
+                         WHERE album_genres.album_id NOT IN (\
+                            SELECT albums.rowid FROM albums)")
+            sql.execute("DELETE FROM album_artists\
+                         WHERE album_artists.album_id NOT IN (\
+                            SELECT albums.rowid FROM albums)")
 
     @property
     def max_count(self):
