@@ -13,22 +13,14 @@
 from gi.repository import GObject, GLib, Gtk, Gdk, GdkPixbuf
 
 from PIL import Image, ImageFilter
+import cairo
 
-from lollypop.define import App
+from lollypop.define import App, ArtHelperEffect
 from lollypop.logger import Logger
 from lollypop.utils import get_round_surface
 from lollypop.information_store import InformationStore
 
 
-class ArtHelperEffect:
-    NONE = 1 << 1
-    ROUNDED = 1 << 2
-    BLUR = 1 << 3
-    BLUR_HARD = 1 << 4
-    RESIZE = 1 << 5
-    FALLBACK = 1 << 6
-
-
 class ArtHelper(GObject.Object):
     """
         Helper to load artwork smoothly
@@ -160,11 +152,27 @@ class ArtHelper(GObject.Object):
             else:
                 surface = Gdk.cairo_surface_create_from_pixbuf(
                         pixbuf, scale_factor, None)
+            if effect & ArtHelperEffect.DARKER:
+                self.__set_color(surface, 0, 0, 0)
         callback(surface)
 
 #######################
 # PRIVATE             #
 #######################
+    def __set_color(self, surface, r, g, b):
+        """
+            Get a darker pixbuf
+            @param surface as cairo.Surface
+            @param r as int
+            @param g as int
+            @param b as int
+            @return cairo.Surface
+        """
+        ctx = cairo.Context(surface)
+        ctx.rectangle(0, 0, surface.get_width(), surface.get_height())
+        ctx.set_source_rgba(r, g, b, 0.5)
+        ctx.fill()
+
     def __get_blur(self, pixbuf, w, h, gaussian):
         """
             Blur surface using PIL
@@ -172,7 +180,7 @@ class ArtHelper(GObject.Object):
             @param w as int
             @param h as int
             @param gaussian as int
-            @return cairo.Surface
+            @return GdkPixbuf.Pixbuf
         """
         if pixbuf is None:
             return None
@@ -210,10 +218,11 @@ class ArtHelper(GObject.Object):
             @param effect as ArtHelperEffect
             @return GdkPixbuf.Pixbuf
         """
-        cache = False if effect & (ArtHelperEffect.BLUR |
-                                   ArtHelperEffect.BLUR_HARD) else True
+        # Do not save blur to cache
+        if not effect & (ArtHelperEffect.BLUR | ArtHelperEffect.BLUR_HARD):
+            effect |= ArtHelperEffect.SAVE
         return App().art.get_album_artwork(album, width, height,
-                                           scale_factor, cache)
+                                           scale_factor, effect)
 
     def __get_radio_artwork(self, radio, width, height, scale_factor, effect):
         """
@@ -225,6 +234,7 @@ class ArtHelper(GObject.Object):
             @param effect as ArtHelperEffect
             @return GdkPixbuf.Pixbuf
         """
+        # Do not save blur to cache
         cache = False if effect & (ArtHelperEffect.BLUR |
                                    ArtHelperEffect.BLUR_HARD) else True
         return App().art.get_radio_artwork(radio, width, height,
@@ -240,6 +250,7 @@ class ArtHelper(GObject.Object):
             @param scale_factor as int
             @return GdkPixbuf.Pixbuf
         """
+        # Do not save blur to cache
         cache = False if effect & (ArtHelperEffect.BLUR |
                                    ArtHelperEffect.BLUR_HARD) else True
         path = InformationStore.get_artwork_path(artist, width,
