@@ -68,69 +68,75 @@ class InformationStore:
             @return path as string/None
         """
         try:
-            filepath = "%s/%s.jpg" % (
-                InformationStore._INFO_PATH,
-                escape(artist))
-            f = Gio.File.new_for_path(filepath)
-            if f.query_exists():
-                info = f.query_info("standard::size",
-                                    Gio.FileQueryInfoFlags.NONE)
-                if info.get_size() == 0:
-                    return None
-                elif not cache:
-                    return filepath
-            else:
-                return None
-            size *= scale_factor
-            extract = None
+            escaped_artist = escape(artist)
             filepath_at_size = "%s/%s_%s.jpg" % (
                 InformationStore._CACHE_PATH,
-                escape(artist),
+                escaped_artist,
                 size)
+            # Search in cache
+            if cache:
+                f = Gio.File.new_for_path(filepath_at_size)
+                if f.query_exists():
+                    return filepath_at_size
+            # Search in store
+            files = ["%s/%s.jpg" % (
+                     InformationStore._INFO_PATH,
+                     escaped_artist),
+                     "%s/web_%s.jpg" % (
+                     InformationStore._INFO_PATH,
+                     escaped_artist)]
+            filepath = None
+            for _filepath in files:
+                f = Gio.File.new_for_path(_filepath)
+                if f.query_exists():
+                    filepath = _filepath
+                    break
+            if filepath is None:
+                return None
+            # Empty image, disabled
+            info = f.query_info("standard::size", Gio.FileQueryInfoFlags.NONE)
+            if info.get_size() == 0:
+                return None
+            # We do not want to cache
+            if not cache:
+                return filepath
+
             # Make cache for this size
-            f = Gio.File.new_for_path(filepath_at_size)
-            if not f.query_exists():
-                pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(filepath,
-                                                                size,
-                                                                size)
-                if pixbuf.get_height() > pixbuf.get_width():
-                    vertical = True
-                elif pixbuf.get_height() < pixbuf.get_width():
-                    vertical = False
+            size *= scale_factor
+            extract = None
+            pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_size(filepath,
+                                                            size,
+                                                            size)
+            if pixbuf.get_height() > pixbuf.get_width():
+                vertical = True
+            elif pixbuf.get_height() < pixbuf.get_width():
+                vertical = False
+            else:
+                extract = pixbuf
+            if extract is None:
+                extract = GdkPixbuf.Pixbuf.new(GdkPixbuf.Colorspace.RGB,
+                                               True, 8, size, size)
+                if vertical:
+                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
+                        filepath,
+                        size,
+                        -1,
+                        True)
+                    diff = pixbuf.get_height() - size
+                    pixbuf.copy_area(0, diff / 2, pixbuf.get_width(), size,
+                                     extract, 0, 0)
                 else:
-                    extract = pixbuf
-                if extract is None:
-                    extract = GdkPixbuf.Pixbuf.new(
-                        GdkPixbuf.Colorspace.RGB,
-                        True, 8,
-                        size, size)
-                    if vertical:
-                        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
-                            filepath,
-                            size,
-                            -1,
-                            True)
-                        diff = pixbuf.get_height() - size
-                        pixbuf.copy_area(0, diff / 2,
-                                         pixbuf.get_width(),
-                                         size,
-                                         extract,
-                                         0, 0)
-                    else:
-                        pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
-                            filepath,
-                            -1,
-                            size,
-                            True)
-                        diff = pixbuf.get_width() - size
-                        pixbuf.copy_area(diff / 2, 0,
-                                         size,
-                                         pixbuf.get_height(),
-                                         extract,
-                                         0, 0)
-                extract.savev(filepath_at_size, "jpeg",
-                              ["quality"], [str(App().settings.get_value(
-                                            "cover-quality").get_int32())])
+                    pixbuf = GdkPixbuf.Pixbuf.new_from_file_at_scale(
+                        filepath,
+                        -1,
+                        size,
+                        True)
+                    diff = pixbuf.get_width() - size
+                    pixbuf.copy_area(diff / 2, 0, size, pixbuf.get_height(),
+                                     extract, 0, 0)
+            extract.savev(filepath_at_size, "jpeg",
+                          ["quality"], [str(App().settings.get_value(
+                                        "cover-quality").get_int32())])
             return filepath_at_size
         except Exception as e:
             Logger.error("InformationStore::get_artwork_path(): %s" % e)
@@ -150,13 +156,20 @@ class InformationStore:
             (status, content, tag) = f.load_contents()
         return content
 
-    def add_artist_artwork(artist, data):
+    def add_artist_artwork(artist, data, is_web=False):
         """
             Add artist artwork to store
             @param artist as str
             @param data as bytes
+            @param is_web as bool
         """
-        filepath = "%s/%s.jpg" % (InformationStore._INFO_PATH, escape(artist))
+        InformationStore.uncache_artwork(artist)
+        if is_web:
+            filepath = "%s/web_%s.jpg" % (InformationStore._INFO_PATH,
+                                          escape(artist))
+        else:
+            filepath = "%s/%s.jpg" % (InformationStore._INFO_PATH,
+                                      escape(artist))
         if data is None:
             f = Gio.File.new_for_path(filepath)
             fstream = f.replace(None, False,
