@@ -12,7 +12,7 @@
 
 import gi
 gi.require_version("Secret", "1")
-from gi.repository import Gio, GLib
+from gi.repository import Gio, GLib, GObject
 
 from gettext import gettext as _
 
@@ -35,7 +35,7 @@ from lollypop.logger import Logger
 from lollypop.goa import GoaSyncedAccount
 
 
-class LastFM(LastFMNetwork, LibreFMNetwork):
+class LastFM(GObject.Object, LastFMNetwork, LibreFMNetwork):
     """
        Lastfm:
        We recommend you don"t distribute the API key and secret with your app,
@@ -46,12 +46,16 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
        want, and if your app isn"t written in a compiled language, you don"t
        really have much option :).
     """
+    __gsignals__ = {
+        "new-artist": (GObject.SignalFlags.RUN_FIRST, None, (str, str)),
+    }
 
     def __init__(self, name):
         """
             Init lastfm support
             @param name as str
         """
+        GObject.Object.__init__(self)
         self.__name = name
         self.__login = ""
         self.session_key = ""
@@ -77,7 +81,7 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             LastFMNetwork.__init__(self,
                                    api_key=self.__API_KEY,
                                    api_secret=self.__API_SECRET)
-        self.connect(False)
+        self.connect()
 
     def connect(self, full_sync=False, callback=None, *args):
         """
@@ -191,20 +195,26 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             except Exception as e:
                 Logger.error("Lastfm::unlove(): %s" % e)
 
-    def get_similars(self, artist):
+    def search_similar_artists(self, artist, cancellable):
         """
-            Get similar artists
+            Search similar artists
             @param artist as str
-            @return artists as [str]
+            @param cancellable as Gio.Cancellable
         """
-        artists = []
         try:
+            found = False
             artist_item = self.get_artist(artist)
             for similar_item in artist_item.get_similar():
-                artists.append(similar_item.item.name)
-        except:
-            pass
-        return artists
+                if cancellable.is_cancelled():
+                    raise Exception("cancelled")
+                found = True
+                artist_name = similar_item.item.name
+                cover_uri = similar_item.item.get_cover_image()
+                GLib.idle_add(self.emit, "new-artist", artist_name, cover_uri)
+        except Exception as e:
+            Logger.error("LastFM::search_similar_artists(): %s", e)
+        if not found:
+            GLib.idle_add(self.emit, "new-artist", None, None)
 
     def on_goa_account_switched(self, obj):
         """
@@ -305,8 +315,7 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
         except Exception as e:
             Logger.debug("LastFM::__connect(): %s" % e)
 
-    def __scrobble(self, artist, album, title, timestamp, mb_track_id,
-                   first=True):
+    def __scrobble(self, artist, album, title, timestamp, mb_track_id):
         """
             Scrobble track
             @param artist as str
@@ -315,7 +324,6 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             @param timestamp as int
             @param duration as int
             @param mb_track_id as str
-            @param first is internal
             @thread safe
         """
         if App().settings.get_value("disable-scrobbling"):
@@ -336,21 +344,14 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             pass
         except Exception as e:
             Logger.error("LastFM::__scrobble(): %s" % e)
-            # Scrobble sometimes fails
-            if first:
-                self.__connect()
-                self.__scrobble(artist, album, title, timestamp,
-                                mb_track_id, False)
 
-    def __now_playing(self, artist, album, title, duration, mb_track_id,
-                      first=True):
+    def __now_playing(self, artist, album, title, duration, mb_track_id):
         """
             Now playing track
             @param artist as str
             @param title as str
             @param album as str
             @param duration as int
-            @param first is internal
             @thread safe
         """
         if App().settings.get_value("disable-scrobbling"):
@@ -367,29 +368,25 @@ class LastFM(LastFMNetwork, LibreFMNetwork):
             pass
         except Exception as e:
             Logger.error("LastFM::__now_playing(): %s" % e)
-            # now playing sometimes fails
-            if first:
-                self.__connect()
-                self.__now_playing(artist, album, title, duration,
-                                   mb_track_id, False)
 
     def __populate_loved_tracks(self):
         """
             Populate loved tracks playlist
         """
-        if not self.available or\
-                App().settings.get_value("lastfm-loved-status"):
+        if not self.available:
             return
         try:
             user = self.get_user(self.__login)
-            for loved in user.get_loved_tracks():
-                track_id = App().tracks.search_track(
-                    str(loved.track.artist),
-                    str(loved.track.title))
-                if track_id is not None:
+            for loved in user.get_loved_tracks(limit=None):
+                artist = str(loved.track.artist)
+                title = str(loved.track.title)
+                track_id = App().tracks.search_track(artist, title)
+                if track_id is None:
+                    Logger.warning(
+                        "LastFM::__populate_loved_tracks(): %s, %s" % (
+                            artist, title))
+                else:
                     Track(track_id).set_loved(1)
-            App().settings.set_value("lastfm-loved-status",
-                                     GLib.Variant("b", True))
         except Exception as e:
             Logger.error("LastFM::__populate_loved_tracks: %s" % e)
 
