@@ -23,7 +23,7 @@ from lollypop.player_playlist import PlaylistPlayer
 from lollypop.radios import Radios
 from lollypop.logger import Logger
 from lollypop.objects import Track, Album
-from lollypop.define import App, Type, NextContext, LOLLYPOP_DATA_PATH, Shuffle
+from lollypop.define import App, Type, LOLLYPOP_DATA_PATH, Shuffle
 
 
 class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
@@ -43,15 +43,13 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         PlaylistPlayer.__init__(self)
         RadioPlayer.__init__(self)
         self.update_crossfading()
-        App().settings.connect("changed::playback", self.__on_playback_changed)
+        App().settings.connect("changed::repeat", self.__on_repeat_changed)
         self._albums_backup = []
 
     def prev(self):
         """
             Play previous track
         """
-        if self._is_locked:
-            return
         if self._prev_track.id is not None:
             if self.position / Gst.SECOND > 2:
                 self.seek(0)
@@ -64,8 +62,6 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         """
             Play next track
         """
-        if self._is_locked:
-            return
         if self._next_track.id is not None:
             self._scrobble(self._current_track, self._start_time)
             self.load(self._next_track)
@@ -286,6 +282,34 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
             self.__play_albums(album, albums)
         self.emit("playlist-changed")
 
+    def play_uris(self, uris):
+        """
+            Play uris
+            @param uri as [str]
+        """
+        # First get tracks
+        tracks = []
+        for uri in uris:
+            track_id = App().tracks.get_id_by_uri(uri)
+            if track_id is not None:
+                tracks.append(Track(track_id))
+        # Then get album ids
+        album_ids = {}
+        for track in tracks:
+            if track.album.id in album_ids.keys():
+                album_ids[track.album.id].append(track)
+            else:
+                album_ids[track.album.id] = [track]
+        # Create albums with tracks
+        play = True
+        for album_id in album_ids.keys():
+            album = Album(album_id)
+            album.set_tracks(album_ids[album_id])
+            if play:
+                self.play_album(album)
+            else:
+                self.add_album(album)
+
     def clear_albums(self):
         """
             Clear all albums
@@ -354,7 +378,7 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
                             for track in tracks:
                                 if track.id == current_track_id:
                                     break
-                        App().player.populate_playlist_by_tracks(
+                        self.populate_playlist_by_tracks(
                             tracks, playlist_ids, track)
                     if is_playing:
                         self.play()
@@ -382,13 +406,10 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         """
             Set previous track
         """
+        if self.current_track.id == Type.RADIOS:
+            return
         try:
-            # Same track
-            if App().settings.get_enum("playback") == NextContext.REPEAT_TRACK:
-                prev_track = self._current_track
-            # Look at shuffle
-            else:
-                prev_track = ShufflePlayer.prev(self)
+            prev_track = ShufflePlayer.prev(self)
 
             # Look at user playlist then
             if prev_track.id is None:
@@ -402,35 +423,30 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         except Exception as e:
             Logger.error("Player::set_prev(): %s" % e)
 
-    def set_next(self, force=False):
+    def set_next(self):
         """
             Play next track
             @param sql as sqlite cursor
-            @param force as bool
         """
+        if self.current_track.id == Type.RADIOS:
+            return
         try:
-            # Reset finished context
-            self._next_context = NextContext.NONE
-
-            # Same track
-            if App().settings.get_enum("playback") == NextContext.REPEAT_TRACK:
-                next_track = self._current_track
-            # Look first at user queue
-            else:
-                next_track = QueuePlayer.next(self)
+            next_track = QueuePlayer.next(self)
 
             # Look at shuffle
             if next_track.id is None:
                 next_track = ShufflePlayer.next(self)
-
             # Look at user playlist then
             if next_track.id is None:
-                next_track = PlaylistPlayer.next(self, force)
+                next_track = PlaylistPlayer.next(self)
 
             # Get a linear track then
             if next_track.id is None:
                 next_track = LinearPlayer.next(self)
+
             self._next_track = next_track
+            if next_track.is_web:
+                App().task_helper.run(self._load_from_web, next_track, False)
             self.emit("next-changed")
         except Exception as e:
             Logger.error("Player::set_next(): %s" % e)
@@ -518,19 +534,14 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         """
             On stream start, set next and previous track
         """
-        if not App().scanner.is_locked():
-            App().window.container.progress.pulse(False)
         if self._current_track.id is not None and self._current_track.id >= 0:
             ShufflePlayer._on_stream_start(self, bus, message)
         if self.track_in_queue(self._current_track):
             self.remove_from_queue(self._current_track.id)
         else:
-            # Do not update next if user clicked on a track
-            if self.is_party and self._current_track != self._next_track:
-                pass
-            else:
-                self.set_next()
-            self.set_prev()
+            self._current_playback_track = self._current_track
+        self.set_prev()
+        self.set_next()
         BinPlayer._on_stream_start(self, bus, message)
 
 #######################
@@ -587,7 +598,7 @@ class Player(BinPlayer, QueuePlayer, PlaylistPlayer, RadioPlayer,
         if track is not None:
             self.load(track)
 
-    def __on_playback_changed(self, settings, value):
+    def __on_repeat_changed(self, settings, value):
         """
             reset next/prev
             @param settings as Gio.Settings, value as str
