@@ -10,14 +10,14 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gst, GstAudio, GstPbutils, GLib
+from gi.repository import Gst, GstAudio, GstPbutils, GLib, Gio
 
 from time import time
 
 from lollypop.player_base import BasePlayer
 from lollypop.tagreader import TagReader
 from lollypop.player_plugins import PluginsPlayer
-from lollypop.define import GstPlayFlags, NextContext, App
+from lollypop.define import GstPlayFlags, App
 from lollypop.codecs import Codecs
 from lollypop.define import Type
 from lollypop.logger import Logger
@@ -34,6 +34,7 @@ class BinPlayer(BasePlayer):
             Init playbin
         """
         BasePlayer.__init__(self)
+        self.__cancellable = Gio.Cancellable()
         self.__codecs = Codecs()
         self._playbin = self.__playbin1 = Gst.ElementFactory.make(
             "playbin", "player")
@@ -107,14 +108,22 @@ class BinPlayer(BasePlayer):
             self._playbin.set_state(Gst.State.PAUSED)
         self.emit("status-changed")
 
-    def stop(self):
+    def stop(self, force=False):
         """
             Change player state to STOPPED
+            @param force as bool
         """
         self._current_track = Track()
         self._playbin.set_state(Gst.State.NULL)
         self.emit("status-changed")
         self.emit("current-changed")
+        if force:
+            self._prev_track = Track()
+            self._next_track = Track()
+            App().player.emit("prev-changed")
+            App().player.emit("next-changed")
+            self._albums = []
+            self.reset_history()
 
     def stop_all(self):
         """
@@ -139,7 +148,7 @@ class BinPlayer(BasePlayer):
             Seek current track to position
             @param position as seconds
         """
-        if self.is_locked or self._current_track.id is None:
+        if self._current_track.id is None:
             return
         # Seems gstreamer doesn"t like seeking to end, sometimes
         # doesn"t go to next track
@@ -210,9 +219,12 @@ class BinPlayer(BasePlayer):
             Set player volume rate
             @param rate as double
         """
+        if rate < 0.0:
+            rate = 0.0
+        elif rate > 1.0:
+            rate = 1.0
         self.__playbin1.set_volume(GstAudio.StreamVolumeFormat.CUBIC, rate)
         self.__playbin2.set_volume(GstAudio.StreamVolumeFormat.CUBIC, rate)
-        self.emit("volume-changed")
 
     def next(self):
         """
@@ -234,13 +246,42 @@ class BinPlayer(BasePlayer):
             self._plugins.volume.props.volume = 1.0
         Logger.debug("BinPlayer::_load_track(): %s" % track.uri)
         try:
+            self.__cancellable.cancel()
+            self.__cancellable.reset()
             self._current_track = track
-            self._playbin.set_property("uri", track.uri)
+            # We check track is URI track, if yes, do a load from Web
+            # Will not work if we add another music provider one day
+            track_uri = App().tracks.get_uri(track.id)
+            if track.is_web and track.uri == track_uri:
+                self.emit("loading-changed", True)
+                App().task_helper.run(self._load_from_web, track)
+                return False
+            else:
+                self._playbin.set_property("uri", track.uri)
         except Exception as e:  # Gstreamer error
             Logger.error("BinPlayer::_load_track(): %s" % e)
             return False
         return True
 
+    def _load_from_web(self, track, play=True):
+        """
+            Load track from web
+            @param track as Track
+            @param play as bool
+        """
+        def play_uri(uri):
+            track.set_uri(uri)
+            if play:
+                self.load(track)
+                App().task_helper.run(self.__update_current_duration,
+                                      track, uri)
+
+        from lollypop.helper_web import WebHelper
+        helper = WebHelper()
+        helper.set_uri(track, self.__cancellable)
+        uri = helper.get_track_content(track)
+        GLib.idle_add(play_uri, uri)
+
     def _scrobble(self, finished, finished_start_time):
         """
             Scrobble on lastfm
@@ -261,10 +302,11 @@ class BinPlayer(BasePlayer):
     def _on_stream_start(self, bus, message):
         """
             On stream start
-            Emit "current-changed" to notify others components
+            Handle stream start: scrobbling, notify, ...
             @param bus as Gst.Bus
             @param message as Gst.Message
         """
+        self.emit("loading-changed", False)
         self._start_time = time()
         Logger.debug("Player::_on_stream_start(): %s" %
                      self._current_track.uri)
@@ -320,29 +362,28 @@ class BinPlayer(BasePlayer):
             @param bus as Gst.Bus
             @param message as Gst.Message
         """
+        self.emit("loading-changed", False)
         Logger.info("Player::_on_bus_error(): %s" % message.parse_error()[1])
-        App().window.container.progress.pulse(False)
-        if self.__codecs.is_missing_codec(message):
-            self.__codecs.install()
-            App().scanner.stop()
-            self.stop()
-        elif App().notify is not None:
-            App().notify.send(message.parse_error()[0].message)
-            self.set_next()
-            self.set_prev()
-            self.next()
+        if self.current_track.id >= 0:
+            if self.__codecs.is_missing_codec(message):
+                self.__codecs.install()
+                App().scanner.stop()
+                self.stop()
+            elif App().notify is not None:
+                App().notify.send(message.parse_error()[0].message)
+                self.stop()
 
     def _on_bus_eos(self, bus, message):
         """
             On end of stream, stop playback
             go next otherwise
         """
+        self.emit("loading-changed", False)
         Logger.debug("Player::__on_bus_eos(): %s" % self._current_track.uri)
         if self._playbin.get_bus() == bus:
-            self._next_context = NextContext.NONE
-            if self._next_track.id is not None:
-                self._load_track(self._next_track)
-            self.emit("current-changed")
+            if App().notify is not None:
+                App().notify.send(message.parse_error()[0].message)
+            self.stop()
 
     def _on_stream_about_to_finish(self, playbin):
         """
@@ -375,12 +416,12 @@ class BinPlayer(BasePlayer):
                     pop_to_add = 1
             App().albums.set_more_popular(self._current_track.album_id,
                                           pop_to_add)
-        if self.__need_to_stop():
+        if self._next_track.id is None:
             # We are in gstreamer thread
             GLib.idle_add(self.stop)
             # Reenable as it has been disabled by do_crossfading()
             self.update_crossfading()
-        elif self._next_track.id is not None:
+        else:
             self._load_track(self._next_track)
 
 #######################
@@ -459,7 +500,9 @@ class BinPlayer(BasePlayer):
             @param next as bool
         """
         # No cossfading if we need to stop
-        if self.__need_to_stop():
+        next_ok = next and self._next_track.id is not None
+        prev_ok = not next and self._prev_track.id is not None
+        if track is None and not next_ok or not prev_ok:
             self._crossfading = False
             return
 
@@ -496,7 +539,7 @@ class BinPlayer(BasePlayer):
             self._plugins.volume.props.volume = 0
             GLib.idle_add(self.__volume_up, self._playbin,
                           self._plugins, duration)
-        elif next and self._next_track.id is not None:
+        elif next:
             self.__load(self._next_track, False)
             self._plugins.volume.props.volume = 0
             GLib.idle_add(self.__volume_up, self._playbin,
@@ -507,20 +550,21 @@ class BinPlayer(BasePlayer):
             GLib.idle_add(self.__volume_up, self._playbin,
                           self._plugins, duration)
 
-    def __need_to_stop(self):
+    def __update_current_duration(self, track, uri):
         """
-            Return True if playback needs to stop
-            @return bool
+            Update current track duration
+            @param track as Track
+            @param uri as str
         """
-        stop = False
-        playback = App().settings.get_enum("playback")
-        if playback == NextContext.STOP:
-            if (not self._albums and
-                not self.queue and
-                not self._playlist_ids) or\
-                    playback == self._next_context:
-                stop = True
-        return stop
+        try:
+            reader = TagReader()
+            duration = reader.get_info(uri).get_duration() / 1000000000
+            if duration != track.duration and duration > 0:
+                App().tracks.set_duration(track.id, int(duration))
+                track.reset("duration")
+                GLib.idle_add(self.emit, "duration-changed", track.id)
+        except Exception as e:
+            Logger.error("BinPlayer::__update_current_duration(): %s" % e)
 
     def __on_volume_changed(self, playbin, sink):
         """
@@ -535,14 +579,3 @@ class BinPlayer(BasePlayer):
             vol = self.__playbin2.get_volume(GstAudio.StreamVolumeFormat.CUBIC)
             self.__playbin1.set_volume(GstAudio.StreamVolumeFormat.CUBIC, vol)
         self.emit("volume-changed")
-
-    def __set_gv_uri(self, uri, track, play):
-        """
-            Play uri for io
-            @param uri as str
-            @param track as Track
-            @param play as bool
-        """
-        track.set_uri(uri)
-        if play:
-            self.load(track)
