@@ -15,7 +15,7 @@ from gi.repository import Gst, GstPbutils, GLib, Gio
 from re import match
 from gettext import gettext as _
 
-from lollypop.define import App, ENCODING
+from lollypop.define import App
 from lollypop.logger import Logger
 from lollypop.utils import format_artist_name
 
@@ -372,6 +372,39 @@ class TagReader(Discoverer):
             values = get_ogg()
         return values
 
+    def get_popm(self, tags):
+        """
+            Get popularity tag
+            @return value as Lollypop rating
+        """
+        if tags is None:
+            return 0
+        size = tags.get_tag_size("private-id3v2-frame")
+        for i in range(0, size):
+            (exists, sample) = tags.get_sample_index("private-id3v2-frame", i)
+            if not exists:
+                continue
+            (exists, m) = sample.get_buffer().map(Gst.MapFlags.READ)
+            if not exists:
+                continue
+            if m.data[0:4] == b"POPM":
+                # Get tag
+                popm = m.data[11]
+                if popm == 0:
+                    value = 0
+                elif popm == 1:
+                    value = 1
+                elif popm == 64:
+                    value = 2
+                elif popm == 128:
+                    value = 3
+                elif popm == 196:
+                    value = 4
+                else:
+                    value = 5
+                return value
+        return 0
+
     def get_lyrics(self, tags):
         """
             Return lyrics for tags
@@ -380,31 +413,14 @@ class TagReader(Discoverer):
             @return lyrics as str
         """
         def decode_lyrics(bytes):
+            from lollypop.utils import decodeUnicode, splitUnicode
             try:
-                lyrics = b""
-                if bytes[0:4] == b"TXXX":
-                    if bytes[13:24] == b"L\x00y\x00r\x00i\x00c\x00s":
-                        lyrics = bytes[29:].replace(b"\x00", b"")
-                    else:
-                        return None
-                elif bytes[0:4] == b"USLT":
-                    # Search for lyrics (more than 10 chars)
-                    for value in bytes.split(b"\x00"):
-                        if len(value) > 10:
-                            lyrics = value
-                            break
-                    # UTF-16
-                    if not lyrics:
-                        lyrics = bytes.split(
-                            b"\xff\xfe")[2].replace(b"\x00", b"")
-                else:
-                    return None
-                for encoding in ENCODING:
-                    try:
-                        return lyrics.decode(encoding)
-                    except Exception as e:
-                        Logger.warning("TagReader::get_lyrics(ENCODING): %s",
-                                       e)
+                prefix = bytes[0:4]
+                if prefix in [b"USLT"]:
+                    frame = bytes[10:]
+                    encoding = frame[0:1]
+                    (d, t) = splitUnicode(frame[4:], encoding)
+                    return decodeUnicode(t, encoding)
             except Exception as e:
                 Logger.warning("TagReader::get_lyrics(): %s", e)
             return None
@@ -546,26 +562,28 @@ class TagReader(Discoverer):
                   uri, loved, popularity, rate, mtime):
         """
             Add album to db
-            @param album name as string
-            @param album artist ids as [int]
-            @param uri to an album track as string
+            @param album name as str
+            @param album artist_ids as [int]
+            @param uri as str
             @param year as int
             @param loved as bool
             @param popularity as int
             @param rate as int
             @param mtime as int
-            @return album_id as int
+            @return (added as bool, album_id as int)
             @commit needed
         """
+        added = False
         f = Gio.File.new_for_uri(uri)
         d = f.get_parent()
-        parent_uri = "/" if d is None else d.get_uri()
+        parent_uri = "" if d is None else d.get_uri()
         album_id = App().albums.get_id(album_name, mb_album_id, artist_ids)
         if album_id is None:
+            added = True
             album_id = App().albums.add(album_name, mb_album_id, artist_ids,
                                         parent_uri, loved, popularity,
                                         rate, mtime)
         # Now we have our album id, check if path doesn"t change
         if App().albums.get_uri(album_id) != parent_uri:
             App().albums.set_uri(album_id, parent_uri)
-        return album_id
+        return (added, album_id)
