@@ -32,7 +32,6 @@ class Toolbar(Gtk.HeaderBar):
         """
         Gtk.HeaderBar.__init__(self)
         self.__width = Sizing.SMALL
-        self.__mini = False
         self.set_title("Lollypop")
         self.__toolbar_playback = ToolbarPlayback(window)
         self.__toolbar_playback.show()
@@ -64,22 +63,19 @@ class Toolbar(Gtk.HeaderBar):
             Calculate infos/title width
             @param window width as int
         """
-        if not self.__mini:
-            width = self.__toolbar_playback.get_preferred_width()[1]
-            width += self.__toolbar_end.get_preferred_width()[1]
-            window = self.get_window()
-            if window is not None:
-                available = window.get_width() - width
-                if available > 0:
-                    if window_width >= Sizing.MEDIUM:
-                        title = available / 2
-                    else:
-                        title = available
-                    self.__toolbar_title.set_width(title)
-                    self.__toolbar_info.set_width((available - title) / 1.5)
-                self.__width = window.get_width()
-        else:
-            self.__toolbar_info.set_width(window_width / 2)
+        width = self.__toolbar_playback.get_preferred_width()[1]
+        width += self.__toolbar_end.get_preferred_width()[1]
+        window = self.get_window()
+        if window is not None:
+            available = window.get_width() - width
+            if available > 0:
+                if window_width >= Sizing.MEDIUM:
+                    title = available / 2
+                else:
+                    title = available
+                self.__toolbar_title.set_width(title)
+                self.__toolbar_info.set_width((available - title) / 1.5)
+            self.__width = window.get_width()
 
     def update_position(self, value=None):
         """
@@ -93,7 +89,6 @@ class Toolbar(Gtk.HeaderBar):
             Set toolbar working when small
             @param mini as bool
         """
-        self.__mini = mini
         self.__toolbar_playback.set_mini(mini)
         self.__toolbar_title.set_mini(mini)
         self.__toolbar_info.set_mini(mini)
