@@ -10,7 +10,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, Gdk, GLib, GObject, Pango
+from gi.repository import Gtk, Gdk, Gio, GLib, GObject, Pango
 
 from gettext import gettext as _
 
@@ -19,9 +19,10 @@ from lollypop.shown import ShownLists
 from lollypop.view_tracks import TracksView
 from lollypop.view import LazyLoadingView
 from lollypop.objects import Album, Track
-from lollypop.define import ArtSize, App, ViewType, MARGIN, MARGIN_SMALL
+from lollypop.define import ArtSize, App, ViewType, MARGIN, MARGIN_SMALL, Type
 from lollypop.controller_view import ViewController, ViewControllerType
 from lollypop.widgets_row_dnd import DNDRow
+from lollypop.logger import Logger
 
 
 class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
@@ -59,7 +60,7 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
         else:
             return cover_height + 2
 
-    def __init__(self, album, height, view_type, reveal, parent):
+    def __init__(self, album, height, view_type, reveal, cover_uri, parent):
         """
             Init row widgets
             @param album as Album
@@ -75,11 +76,12 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
         self.__revealer = None
         self.__parent = parent
         self.__reveal = reveal
+        self.__cover_uri = cover_uri
         self._artwork = None
+        self.__cover_data = None
         self._album = album
-        self.__x_root = self.__y_root = 0
         self.__view_type = view_type
-        self.__play_indicator = None
+        self.__cancellable = Gio.Cancellable()
         self.set_sensitive(False)
         self.set_property("height-request", height)
         self.connect("destroy", self.__on_destroy)
@@ -100,7 +102,6 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
         # and down on reveal()
         self._artwork.set_margin_top(self.__MARGIN)
         self.get_style_context().add_class("albumrow")
-        self.get_style_context().add_class("albumrow-hover")
         self.set_sensitive(True)
         self.set_selectable(False)
         self.set_property("has-tooltip", True)
@@ -121,7 +122,6 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
         self.__title_label.set_ellipsize(Pango.EllipsizeMode.END)
         self.__title_label.set_property("halign", Gtk.Align.START)
         self.__title_label.get_style_context().add_class("dim-label")
-        self.set_artwork()
         self.__action_button = None
         if self.__view_type & ViewType.DND:
             self.__action_button = Gtk.Button.new_from_icon_name(
@@ -129,6 +129,11 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
                 Gtk.IconSize.MENU)
             self.__action_button.set_tooltip_text(
                 _("Remove from current playlist"))
+        elif self._album.mtime == 0:
+            self.__action_button = Gtk.Button.new_from_icon_name(
+                "document-save-symbolic",
+                Gtk.IconSize.MENU)
+            self.__action_button.set_tooltip_text(_("Save in collection"))
         elif self.__view_type & ViewType.SEARCH:
             self.__action_button = Gtk.Button.new_from_icon_name(
                     'avatar-default-symbolic',
@@ -166,7 +171,14 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
         self.__gesture.set_propagation_phase(Gtk.PropagationPhase.CAPTURE)
         self.__gesture.set_button(0)
         if self.__reveal:
-            self.reveal()
+            self.reveal(True)
+        if self.__cover_uri is None:
+            self.set_artwork()
+        else:
+            self.__on_album_artwork(None)
+            App().task_helper.load_uri_content(self.__cover_uri,
+                                               self.__cancellable,
+                                               self.__on_cover_uri_content)
 
     def append_rows(self, tracks):
         """
@@ -184,20 +196,14 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
         if self._responsive_widget is not None:
             TracksView.prepend_rows(self, tracks)
 
-    def reveal(self, reveal=None,
-               transition_type=Gtk.RevealerTransitionType.SLIDE_DOWN):
+    def reveal(self, reveal=None):
         """
             Reveal/Unreveal tracks
-            @param revleal as bool or None to just change state
-            @param transition_type as Gtk.RevealerTransitionType
+            @param reveal as bool or None to just change state
         """
-        self.__revealer.set_transition_type(transition_type)
         if self.__revealer.get_reveal_child() and reveal is not True:
             self.__revealer.set_reveal_child(False)
-            if self.__action_button is not None:
-                self.__action_button.set_opacity(1)
-                self.__action_button.set_sensitive(True)
-            self.get_style_context().add_class("albumrow-hover")
+            self.get_style_context().remove_class("albumrow-hover")
             if self.album.id == App().player.current_track.album.id:
                 self.set_state_flags(Gtk.StateFlags.VISITED, True)
         else:
@@ -205,12 +211,9 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
                 TracksView.populate(self)
                 self._responsive_widget.show()
                 self.__revealer.add(self._responsive_widget)
-            self.get_style_context().remove_class("albumrow-hover")
             self.__revealer.set_reveal_child(True)
-            if self.__action_button is not None:
-                self.__action_button.set_opacity(0)
-                self.__action_button.set_sensitive(False)
-            self.set_state_flags(Gtk.StateFlags.NORMAL, True)
+            self.unset_state_flags(Gtk.StateFlags.VISITED)
+            self.get_style_context().add_class("albumrow-hover")
 
     def set_playing_indicator(self):
         """
@@ -240,6 +243,8 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
         """
             Set album artwork
         """
+        if self._artwork is None:
+            return
         App().art_helper.set_album_artwork(self._album,
                                            ArtSize.MEDIUM,
                                            ArtSize.MEDIUM,
@@ -319,7 +324,7 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
             self.get_style_context().remove_class("track-menu-selected")
 
         from lollypop.pop_menu import AlbumMenu
-        menu = AlbumMenu(self._album, True)
+        menu = AlbumMenu(self._album, ViewType.ALBUM)
         popover = Gtk.Popover.new_from_model(widget, menu)
         popover.connect("closed", on_closed)
         self.get_style_context().add_class("track-menu-selected")
@@ -331,6 +336,21 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
             popover.set_pointing_to(rect)
         popover.popup()
 
+    def __on_cover_uri_content(self, uri, status, data):
+        """
+            Save to tmp cache
+            @param uri as str
+            @param status as bool
+            @param data as bytes
+        """
+        try:
+            if status:
+                self.__cover_data = data
+                App().art.save_album_artwork(self.__cover_data, self._album.id)
+                self.set_artwork()
+        except Exception as e:
+            Logger.error("AlbumRow::__on_cover_uri_content(): %s", e)
+
     def __on_album_artwork(self, surface):
         """
             Set album artwork
@@ -388,23 +408,23 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
 
     def __on_action_button_release_event(self, button, event):
         """
-            ViewType.SEARCH: Play album
-            Else: Delete album
+            Handle button actions
             @param button as Gtk.Button
             @param event as Gdk.Event
         """
         if not self.get_state_flags() & Gtk.StateFlags.PRELIGHT:
-            return
-        if self.__view_type & ViewType.SEARCH:
+            return True
+        if self._album.mtime == 0:
+            self._album.save(True)
+            if self.__cover_data is not None:
+                App().art.save_album_artwork(self.__cover_data, self._album.id)
+            App().art.cache_artists_info()
+            self.destroy()
+        elif self.__view_type & ViewType.SEARCH:
             popover = self.get_ancestor(Gtk.Popover)
             if popover is not None:
                 popover.popdown()
-            if App().settings.get_value("show-sidebar") and\
-                    not App().window.is_adaptive:
-                App().window.container.show_artists_albums(
-                    self._album.artist_ids)
-            else:
-                App().window.container.show_view(self._album.artist_ids[0])
+            App().window.container.show_artist_view(self._album.artist_ids)
         elif self.__view_type & ViewType.DND:
             if App().player.current_track.album.id == self._album.id:
                 # If not last album, skip it
@@ -445,6 +465,7 @@ class AlbumRow(Gtk.ListBoxRow, TracksView, DNDRow):
             Destroyed widget
             @param widget as Gtk.Widget
         """
+        self.__cancellable.cancel()
         self._artwork = None
 
 
@@ -465,7 +486,11 @@ class AlbumsListView(LazyLoadingView, ViewController):
         self.__genre_ids = genre_ids
         self.__artist_ids = artist_ids
         if genre_ids and genre_ids[0] < 0:
-            self._empty_message = ShownLists.IDS[genre_ids[0]][0]
+            if genre_ids[0] == Type.WEB and\
+                    GLib.find_program_in_path("youtube-dl") is None:
+                self._empty_message = _("Missing youtube-dl command")
+            else:
+                self._empty_message = ShownLists.IDS[genre_ids[0]][0]
             self._empty_icon_name = get_icon_name(genre_ids[0])
         self.__autoscroll_timeout_id = None
         self.__reveals = []
@@ -491,6 +516,26 @@ class AlbumsListView(LazyLoadingView, ViewController):
         """
         self.__reveals = album_ids
 
+    def insert_album(self, album, reveal, position, cover_uri=None):
+        """
+            Add an album
+            @param album as Album
+            @param reveal as bool
+            @param position as int
+            @param cover_uri as str
+        """
+        row = self.__row_for_album(album, reveal, cover_uri)
+        children = self._box.get_children()
+        if children:
+            previous_row = children[position]
+            row.set_previous_row(previous_row)
+            previous_row.set_next_row(row)
+        row.populate()
+        row.show()
+        self._box.insert(row, position)
+        if self._viewport.get_child() is None:
+            self._viewport.add(self._box)
+
     def populate(self, albums):
         """
             Populate widget with album rows
@@ -593,6 +638,37 @@ class AlbumsListView(LazyLoadingView, ViewController):
             if child.album.id == album_id:
                 child.set_artwork()
 
+    def _on_duration_changed(self, player, track_id):
+        """
+            Update track duration
+            @param player as Player
+            @param track_id as int
+        """
+        for child in self.children:
+            child.update_duration(track_id)
+
+    def _on_album_updated(self, scanner, album_id, added):
+        """
+            Handles changes in collection
+            @param scanner as CollectionScanner
+            @param album_id as int
+            @param added as bool
+        """
+        if self._view_type & ViewType.SEARCH:
+            return
+        if added:
+            album_ids = App().window.container.get_view_album_ids(
+                                            self.__genre_ids,
+                                            self.__artist_ids)
+            if album_id not in album_ids:
+                return
+            index = album_ids.index(album_id)
+            self.insert_album(Album(album_id), False, index)
+        else:
+            for child in self._box.get_children():
+                if child.album.id == album_id:
+                    child.destroy()
+
     def _on_map(self, widget):
         """
             Connect signals and set active ids
@@ -653,13 +729,15 @@ class AlbumsListView(LazyLoadingView, ViewController):
             if self._viewport.get_child() is None:
                 self._viewport.add(self._box)
 
-    def __row_for_album(self, album, reveal=False):
+    def __row_for_album(self, album, reveal=False, cover_uri=None):
         """
             Get a row for track id
             @param album as Album
             @param reveal as bool
+            @param cover_uri as str
         """
-        row = AlbumRow(album, self.__height, self._view_type, reveal, self)
+        row = AlbumRow(album, self.__height, self._view_type,
+                       reveal, cover_uri, self)
         row.connect("insert-track", self.__on_insert_track)
         row.connect("insert-album", self.__on_insert_album)
         row.connect("insert-album-after", self.__on_insert_album_after)
