@@ -12,8 +12,8 @@
 
 from gi.repository import Gtk, GLib
 
-from lollypop.helper_art import ArtHelperEffect
 from lollypop.define import App, ArtSize, MARGIN, Type, ViewType
+from lollypop.define import ArtHelperEffect
 from lollypop.widgets_rating import RatingWidget
 from lollypop.widgets_loved import LovedWidget
 from lollypop.widgets_cover import CoverWidget
@@ -76,7 +76,6 @@ class AlbumBannerWidget(Gtk.Bin):
             self.__year_label.set_text(str(album.year))
         else:
             self.__year_label.hide()
-        self.__year_label.set_margin_end(MARGIN)
         year_eventbox = builder.get_object("year_eventbox")
         year_eventbox.connect("realize", on_realize)
         year_eventbox.connect("button-release-event",
@@ -88,7 +87,7 @@ class AlbumBannerWidget(Gtk.Bin):
         self.__grid = builder.get_object("grid")
         self.__widget = builder.get_object("widget")
         if view_type & ViewType.ALBUM:
-            self.__grid.get_style_context().add_class("black")
+            self.get_style_context().add_class("black")
             self.__artwork.get_style_context().add_class("black")
             self.connect("size-allocate", self.__on_size_allocate)
             self.connect("destroy", self.__on_destroy)
@@ -97,31 +96,37 @@ class AlbumBannerWidget(Gtk.Bin):
                                                self.__on_album_artwork_changed)
         else:
             self.__grid.get_style_context().add_class("banner-frame")
-            self.__artwork.get_style_context().add_class("banner-frame-back")
             # See application.css: cover-frame
             self.__padding = 8
         if view_type & ViewType.SMALL or not view_type & ViewType.ALBUM:
-            self.__cover_widget = CoverWidget(True, ArtSize.LARGE)
+            self.__cover_widget = CoverWidget(album, ArtSize.LARGE)
         else:
-            self.__cover_widget = CoverWidget(True)
-        self.__cover_widget.update(album)
-        self.__cover_widget.set_margin_start(MARGIN)
+            self.__cover_widget = CoverWidget(album)
         self.__cover_widget.set_vexpand(True)
         self.__cover_widget.show()
         self.__grid.attach(self.__cover_widget, 0, 0, 1, 3)
         self.__rating_grid = builder.get_object("rating_grid")
+        if album.mtime <= 0:
+            cloud = Gtk.Image.new_from_icon_name("goa-panel-symbolic",
+                                                 icon_size)
+            cloud.show()
+            cloud.set_margin_start(MARGIN)
+            self.__rating_grid.attach(cloud, 0, 0, 1, 1)
         rating = RatingWidget(album, icon_size)
         rating.set_property("halign", Gtk.Align.START)
         rating.set_property("valign", Gtk.Align.CENTER)
         rating.show()
-        self.__rating_grid.attach(rating, 0, 0, 1, 1)
+        self.__rating_grid.attach(rating, 1, 0, 1, 1)
         loved = LovedWidget(album, icon_size)
         loved.set_margin_start(10)
         loved.set_property("halign", Gtk.Align.START)
         loved.set_property("valign", Gtk.Align.CENTER)
         loved.show()
-        self.__rating_grid.attach(loved, 1, 0, 1, 1)
+        self.__rating_grid.attach(loved, 2, 0, 1, 1)
         self.add(self.__widget)
+        self.__cover_widget.set_margin_start(MARGIN)
+        self.__year_label.set_margin_end(MARGIN)
+        self.__duration_label.set_margin_end(MARGIN)
 
     def set_height(self, height):
         """
@@ -155,8 +160,12 @@ class AlbumBannerWidget(Gtk.Bin):
             self.__rating_grid.show()
             self.__year_label.set_vexpand(False)
             if not self.__view_type & ViewType.SMALL:
+                self.__title_label.get_style_context().remove_class(
+                    "text-x-large")
                 self.__title_label.get_style_context().add_class(
                     "text-xx-large")
+                self.__year_label.get_style_context().remove_class(
+                    "text-large")
                 self.__year_label.get_style_context().add_class(
                     "text-x-large")
 
@@ -210,7 +219,7 @@ class AlbumBannerWidget(Gtk.Bin):
             @param button as Gtk.Button
         """
         from lollypop.pop_menu import AlbumMenu
-        menu = AlbumMenu(self.__album, True)
+        menu = AlbumMenu(self.__album, self.__view_type)
         popover = Gtk.Popover.new_from_model(button, menu)
         popover.popup()
 
@@ -228,11 +237,14 @@ class AlbumBannerWidget(Gtk.Bin):
         self.__width = allocation.width
         App().art_helper.set_album_artwork(
                 self.__album,
-                allocation.width - self.__padding,
-                allocation.height - self.__padding,
+                # +100 to prevent resize lag
+                allocation.width + 100,
+                allocation.height,
                 self.__artwork.get_scale_factor(),
                 self.__on_album_artwork,
-                ArtHelperEffect.RESIZE | ArtHelperEffect.BLUR_HARD)
+                ArtHelperEffect.RESIZE |
+                ArtHelperEffect.BLUR_HARD |
+                ArtHelperEffect.DARKER)
 
     def __on_destroy(self, widget):
         """
@@ -251,21 +263,21 @@ class AlbumBannerWidget(Gtk.Bin):
         if album_id == self.__album.id:
             App().art_helper.set_album_artwork(
                             self.__album,
-                            self.get_allocated_width() - self.__padding,
-                            self.get_allocated_height() - self.__padding,
+                            # +100 to prevent resize lag
+                            self.get_allocated_width() + 100,
+                            self.get_allocated_height(),
                             self.__artwork.get_scale_factor(),
                             self.__on_album_artwork,
-                            ArtHelperEffect.RESIZE | ArtHelperEffect.BLUR_HARD)
+                            ArtHelperEffect.RESIZE |
+                            ArtHelperEffect.BLUR_HARD |
+                            ArtHelperEffect.DARKER)
 
     def __on_album_artwork(self, surface):
         """
             Set album artwork
             @param surface as str
         """
-        if surface is None:
-            self.__grid.get_style_context().remove_class("black")
-            self.__grid.get_style_context().add_class("black-non-transparent")
-        else:
+        if surface is not None:
             self.__artwork.set_from_surface(surface)
 
     def __on_year_button_release_event(self, widget, event):
@@ -280,7 +292,7 @@ class AlbumBannerWidget(Gtk.Bin):
 
     def __on_size_allocate(self, widget, allocation):
         """
-            Change box max/min children
+            Delayed handling
             @param widget as Gtk.Widget
             @param allocation as Gtk.Allocation
         """
