@@ -63,6 +63,7 @@ class AlbumSimpleWidget(Gtk.FlowBoxChild, AlbumWidget, OverlayAlbumHelper):
             Populate widget content
         """
         OverlayAlbumHelper.__init__(self)
+        self._watch_loading = self._album.mtime <= 0
         self.set_property("halign", Gtk.Align.CENTER)
         self.set_property("valign", Gtk.Align.CENTER)
         self.__widget = Gtk.EventBox()
@@ -102,7 +103,7 @@ class AlbumSimpleWidget(Gtk.FlowBoxChild, AlbumWidget, OverlayAlbumHelper):
         self._overlay.add(self._artwork)
         grid.add(self._overlay)
         grid.add(eventbox)
-        self.set_artwork(self.__art_size, self.__art_size)
+        self.set_artwork()
         self.set_selection()
         if not self.__view_type & (ViewType.SMALL | ViewType.MEDIUM):
             self.__widget.connect("enter-notify-event", self._on_enter_notify)
@@ -112,6 +113,16 @@ class AlbumSimpleWidget(Gtk.FlowBoxChild, AlbumWidget, OverlayAlbumHelper):
         self.connect("destroy", self.__on_destroy)
         self.add(self.__widget)
 
+    def set_artwork(self):
+        """
+            Set artwork
+        """
+        App().art_helper.set_album_artwork(self._album,
+                                           self.__art_size,
+                                           self.__art_size,
+                                           self._artwork.get_scale_factor(),
+                                           self.__on_album_artwork)
+
     def do_get_preferred_width(self):
         """
             Return preferred width
@@ -145,12 +156,24 @@ class AlbumSimpleWidget(Gtk.FlowBoxChild, AlbumWidget, OverlayAlbumHelper):
             Set overlay
             @param show_overlay as bool
         """
-        if self._lock_overlay or\
-                self._show_overlay == show_overlay or\
-                (App().player.is_locked and show_overlay):
+        if self._lock_overlay or self._show_overlay == show_overlay:
             return
         OverlayAlbumHelper._show_overlay_func(self, show_overlay)
         if show_overlay:
+            # Play button
+            self.__play_button = Gtk.Button.new_from_icon_name(
+                "media-playback-start-symbolic",
+                Gtk.IconSize.INVALID)
+            self.__play_button.get_image().set_pixel_size(self._pixel_size +
+                                                          20)
+            self.__play_button.set_property("has-tooltip", True)
+            self.__play_button.set_tooltip_text(_("Play"))
+            self.__play_button.connect("realize", on_realize)
+            self.__play_button.connect("clicked", self.__on_play_clicked)
+            self.__play_button.show()
+            self._big_grid.add(self.__play_button)
+            self.__play_button.get_style_context().add_class(
+                "overlay-button-rounded")
             # Play all button
             self.__play_all_button = Gtk.Button.new()
             self.__play_all_button.set_property("has-tooltip", True)
@@ -166,35 +189,21 @@ class AlbumSimpleWidget(Gtk.FlowBoxChild, AlbumWidget, OverlayAlbumHelper):
             self.__play_all_button.get_style_context().add_class(
                 "overlay-button")
         else:
+            self.__play_button.destroy()
+            self.__play_button = None
             self.__play_all_button.destroy()
             self.__play_all_button = None
 
-    def _on_album_updated(self, scanner, album_id, destroy):
+    def _on_album_updated(self, scanner, album_id, added):
         """
             On album modified, disable it
             @param scanner as CollectionScanner
             @param album id as int
-            @param deleted as bool
-            @param destroy as bool
+            @param added as bool
         """
-        if self._album.id == album_id and destroy:
+        if self._album.id == album_id and not added:
             self.destroy()
 
-    def _on_album_artwork(self, surface):
-        """
-            Set album artwork
-            @param surface as str
-        """
-        if self.__widget is None:
-            return
-        if surface is None:
-            self._artwork.set_from_icon_name("folder-music-symbolic",
-                                             Gtk.IconSize.DIALOG)
-        else:
-            self._artwork.set_from_surface(surface)
-        self.show_all()
-        self.emit("populated")
-
 #######################
 # PRIVATE             #
 #######################
@@ -211,13 +220,42 @@ class AlbumSimpleWidget(Gtk.FlowBoxChild, AlbumWidget, OverlayAlbumHelper):
                 "media-playlist-shuffle-symbolic",
                 Gtk.IconSize.INVALID)
 
+    def __on_album_artwork(self, surface):
+        """
+            Set album artwork
+            @param surface as str
+        """
+        if self.__widget is None:
+            return
+        if surface is None:
+            if self.__art_size == ArtSize.BIG:
+                icon_size = Gtk.IconSize.DIALOG
+            else:
+                icon_size = Gtk.IconSize.DIALOG.DND
+            self._artwork.set_from_icon_name("folder-music-symbolic",
+                                             icon_size)
+        else:
+            self._artwork.set_from_surface(surface)
+        self.show_all()
+        self.emit("populated")
+
+    def __on_play_clicked(self, button):
+        """
+            Play album
+            @param button as Gtk.Button
+        """
+        if App().player.is_party:
+            action = App().lookup_action("party")
+            action.change_state(GLib.Variant("b", False))
+        App().player.play_album(self._album.clone(True))
+        self._show_append(False)
+        return True
+
     def __on_play_all_clicked(self, button):
         """
             Play album with context
             @param button as Gtk.Button
         """
-        if App().player.is_locked:
-            return True
         self._show_append(False)
         if App().player.is_party:
             App().lookup_action("party").change_state(GLib.Variant("b", False))
@@ -232,7 +270,7 @@ class AlbumSimpleWidget(Gtk.FlowBoxChild, AlbumWidget, OverlayAlbumHelper):
             @param eventbox as Gtk.EventBox
             @param event as Gdk.EventButton
         """
-        App().window.container.show_artists_albums(self._album.artist_ids)
+        App().window.container.show_artist_view(self._album.artist_ids)
         return True
 
     def __on_destroy(self, widget):
